package com.zhuangzi.bootdemo01.service;

import com.zhuangzi.bootdemo01.entity.Sorts;
import com.zhuangzi.bootdemo01.repository.SortsRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;

@Service("sortsService")
public class SortsServiceImpl implements SortsService {
    @Autowired
    private SortsRepository sortsDao;
    private static ArrayList<Sorts> all;  //存储所有的查询子类数据
    @Override
    //parentId:  父类 ID, 查询当前类别下的所有子类
    //space :  // 类别前面的空隔
    public  ArrayList<Sorts> getChildren(int parentId, String  space, ArrayList<Sorts> arr){
        if(arr == null){
            //第一次查询
            System.out.println(parentId);
            arr = sortsDao.selectByParentId(parentId);
            all = new ArrayList<>(); //第一次时， 初始化， 避免刷新调用重新添加信息
        }
        if(arr != null){
            for(Sorts sorts : arr){
                //根据级别不一样, 在前面加上相应的分隔符
                int levels = sorts.getLevels();
                if(levels == 1){
                    sorts.setSpace("");
                }
                else if(levels == 2){
                    space = "　　|---->";
                    sorts.setSpace(space);
                }
                else{
                    sorts.setSpace(space);
                }
                all.add(sorts); //将所有的信息都加到all里， 并增加一个space字段
                parentId = sorts.getId();
                ArrayList<Sorts> arr_child = sortsDao.selectByParentId(parentId);
                if(arr_child != null){
                    getChildren(parentId,"　　|" + space,arr_child);
                }
            }
        }
        return  all;
    }

    @Override
    //得到无限分类的select下拉列表
    //pid : 父id
    //selectName select的名字
    //currenId : 被选择的id, 没有的话,传递0 
    public StringBuffer selectTrees(int pid, String selectName, int currentId) {
        StringBuffer sBuffer = new StringBuffer();
        ArrayList<Sorts> results = getChildren(pid,"",null);
        if(results != null){
            sBuffer.append("<select name='"+selectName+"'>\n");
            sBuffer.append("<option value='0'>一级类别</option> \n");
            for(Sorts sorts : results){
                int id = sorts.getId();
                if( id == currentId){
                    sBuffer.append("<option value='"+id+"' style='background:#E20A0A; color:#fff;' selected>"+
                            sorts.getSpace() + sorts.getSortName() + "</option> \n");
                }
                else{
                    sBuffer.append("<option value='"+id+"'>"+ sorts.getSpace() + sorts.getSortName() + "</option> \n");
                }
            }
            sBuffer.append("</select>\n");
        }

        return  sBuffer;
    }

    @Override
    public String addSorts(Sorts sorts) {
        //判断类名不能为空
        if(sorts.getSortName().equals("")) {
            return "<script>alert('类别名称不能为空');history.back();</script>";
        }
        //判断同一级下不能重复的名称
        if(sortsDao.selectBySortNameAndParendId(sorts) != null){
            return "<script>alert('父类下已存在相同类别!!!');history.back();</script>";
        }
        //设置level, sortpath 信息  begin
        sorts.setLevels(1);
        sorts.setSortPath("0,");
        if(sorts.getParentId() != 0){
            //最顶级时使用默认设置值， 否则根据父级来决定
            Sorts getSorts = sortsDao.getOne(sorts.getParentId());
            sorts.setLevels(getSorts.getLevels() + 1);
            sorts.setSortPath(getSorts.getSortPath());
            //暂时为父级sortPath, 当添加后， 再更新，将当前的id链接上
        }

        //将信息添加到数据库sorts
        sortsDao.save(sorts); //sorts中会更新为新增加的id主键

        //更新sortPath 到数据库
        sorts.setSortPath(sorts.getSortPath() + sorts.getId() + ",");
        sortsDao.save(sorts);
        return "<script>alert('添加成功');location.href='/admin/sorts/list';</script>";
    }

    @Override
    public String updateSorts(Sorts sorts) {
        //判断类名不能为空
        if(sorts.getSortName().equals("")) {
            return "<script>alert('类别名称不能为空');history.back();</script>";
        }
        //根据id得到原来的信息
        Sorts oldSorts = sortsDao.getOne(sorts.getId());
        int oldPid = oldSorts.getParentId(); //原父id
        String oldSortPath = oldSorts.getSortPath();
        //判断类别名称是否重复
        Sorts exitSorts = sortsDao.selectSortsByParentIdSortName(sorts);
        if(exitSorts != null){
            return "<script>alert('类别已经存在,请更换类别名称');history.back();</script>";
        }

        if(oldPid == sorts.getParentId()){
            //父id不变, 则parentid, level都不需要修改
            sorts.setSortPath(oldSorts.getSortPath());
            sorts.setLevels(oldSorts.getLevels());
            sortsDao.save(sorts);
        }
        else if(sorts.getParentId() == 0){
            //移动到一级类别
            sorts.setSortPath("0," + sorts.getId() + ",");
            sorts.setLevels(1);
            //更新当前类别信息
            sortsDao.save(sorts);

            //更新所有的修改类别的子类sortPath
            Sorts sort1 = new Sorts();
            sort1.setOldSortPath(oldSortPath);
            sort1.setNewSortPath(sorts.getSortPath());
            sort1.setId(sorts.getId());
            sortsDao.updateAllChildSorts(sort1);
            //更新所有子类的层级
            sortsDao.updateAllChildSortsLevels(sort1);
        }
        else{
            //判断是否移到到了子类另， 这样是不允许的， 中间会断层
            //得到当前父类的path
            Sorts parentSorts = sortsDao.getOne(sorts.getParentId());
            String parentSortPath = parentSorts.getSortPath();
            if(parentSortPath.indexOf(oldSortPath) >= 0){
                return "<script>alert('类别不能选择为原父类的子类');history.back();</script>";
            }
            else{
                //父类更新到上一级或者其它的类别下, 那么该类别下所有的子类都需要更新父级sortpath
                //新的sortpath
                String newSortPath = parentSortPath + sorts.getId() + ",";
                int    newLevel    = parentSorts.getLevels() + 1;
                sorts.setSortPath(newSortPath);
                sorts.setLevels(newLevel);
                sortsDao.save(sorts);
                //更新所有的修改类别的子类

                //更新所有的修改类别的子类sortPath
                //更新所有的修改类别的子类sortPath
                Sorts sort1 = new Sorts();
                sort1.setOldSortPath(oldSortPath);
                sort1.setNewSortPath(sorts.getSortPath());
                sort1.setId(sorts.getId());
                sortsDao.updateAllChildSorts(sort1);
                //更新所有子类的层级
                sortsDao.updateAllChildSortsLevels(sort1);
            }
        }
        return "<script>alert('修改成功');location.href='/admin/sorts/list';</script>";
    }

    //删除所有的子类
    @Override
    public String deleteAllSortsById(Integer id) {
        sortsDao.deleteAllSortsById(id);
        return "<script>alert('删除成功');location.href='/admin/sorts/list';</script>";
    }
}
